from bs4 import BeautifulSoup

from .extractors.extract_feeds import extract_feeds
from .extractors.extract_images import extract_images
from .extractors.extract_links import extract_links
from .extractors.extract_schemas import extract_schemas
from .extractors.extract_src import extract_src
from .extractors.extract_text import extract_text
from .extractors.extract_title import extract_title



def get(url, html):
    return process_extraction(url, html)

def kraken_extract_from_html(url, html):
    return process_extraction(url, html)


def process_extraction(url, html, image_urls = None):

    records = []

    
    text = extract_text(url, html)
    records+=text
    
    soup = _get_soup(html)


    links = extract_links(url, soup)
    records+=links


    images = extract_images(url, soup)
    records += images


    feeds = extract_feeds(url, soup)
    records += feeds

    schemas = extract_schemas(url, html)
    records += schemas

    src = extract_src(url, soup)
    records += src

    texts = extract_text(url, html)
    records += texts

    titles = extract_title(url, soup)
    records += titles


    return records




def _get_soup(html):

    soup = BeautifulSoup(html, 'html.parser')

    return soup










def get(url, urls):
    return extract_urls(url, urls)



def extract_urls(url, urls):
    
    if not isinstance(urls, list):
        urls = [urls]


    schemas = []

    for i in urls:
        new_i = i
        if new_i.startswith('//'):
            new_i = 'https:' + new_i

        new_i = new_i.lower()
        file_name = new_i.split('?')


        image_list = ['.jpeg', '.jpg', '.gif', '.png']
        video_list = ['.mp4', '.mpg', '.mpeg', '.mov']
        
        if any(file_name[0].endswith(s) for s in image_list):
            record = {
                '@type': 'schema:imageObject',
                'schema:url': url,
                'schema:contentUrl': new_i
                }

            schemas.append(record)

        elif any(file_name[0].endswith(s) for s in video_list):
            record = {
            '@type': 'schema:videoObject',
            'schema:url': url,
            'schema:contentUrl': new_i
            }

            schemas.append(record)

        else:
            #todo returns too many schemas
            a=1
            '''
            record = {
            '@type': 'schema:digitalDocument',
            'schema:url': url
            }

            schemas.append(record)
            '''

    return schemas






#from ioc_finder import find_iocs


def get(url, html):
    return extract_iocs(url, html)



def extract_iocs(url, html):

    a=1 
    '''
    iocs = find_iocs(html)

    schemas = []

    schemas += _extract_urls(url, iocs['urls'])
    schemas += _extract_emails(iocs['email_addresses'])

    return schemas

    '''




def get(url, soup):
    return extract_title(url, soup)



def extract_title(url, soup):
    
    if soup.title:
        title = soup.title.string


        record = {
            '@type': 'schema:WebPage',
            'schema:url': url,
            'schema:headline': title
        }


        return [record]
    return []







import urllib.parse



def get(url, soup):
    return extract_feeds(url, soup)


def extract_feeds(url, soup):

    feeds = []

    ref_feeds = soup.findAll(type='application/rss+xml') + soup.findAll(type='application/atom+xml')

    for feed in ref_feeds:

        feed_url = feed.get('href', None)

        # Convert to absolute url
        feed_url = urllib.parse.urljoin(url, feed_url)

        # Compile into structured record
        feed_record = {
            '@type': 'schema:datafeed',
            'schema:title': feed.get('title', None),
            'schema:url': feed_url
        }
        
        # Skip if already in list
        if feed_record  in feeds:
            continue

        # Add to list if url exist
        if feed_record.get('schema:url', None):
            feeds.append(feed_record)
    
    return feeds



import extruct
from w3lib.html import get_base_url


def get(url, html):
    return extract_schemas(url, html)


def extract_schemas(url, html):
    """ Extract schemas from webpage
    """

    schemas = []

    base_url = get_base_url(html, url)

    data = extruct.extract(html, base_url=base_url, uniform=True)

    for i in data:

        for p in data[i]:

            schemas.append(p)

    return schemas
